import { Avatar, Button, Typography } from "@material-tailwind/react";
import {
  BASE_URL,
  USER_CONNECTED,
} from "../../../core/application/constant/common";
import { convertToFrenchDay1 } from "../../../core/application/helpers/dateTime-converter";
import defaultImage from "../../../core/assets/images/event.png";
import { EVENT_TYPES } from "../../infrastructure/constant/list-constant-agenda";
import { Tooltip } from "react-tooltip";
import CoverAndProfilImage from "../../../core/presentation/component/CoverAndProfilImage";
import { ChipWithAvatar } from "../../../core/presentation/component/ChipWithAvatar";
import CardTile from "../../../core/presentation/component/CardTile";
import MiniCardIcon from "../../../core/presentation/component/MiniCardIcon";
import DateRenderer from "../../../core/presentation/component/DateRenderer";
import CircleAvatarStack from "../../../core/presentation/component/CircleAvatarStack";
import PropTypes from 'prop-types';

function EventDetails({ event, mode,  joinEvent, leaveEvent }) {
  const startDate = new Date(event.startDate);
  const endDate = new Date(event.endDate);

  const eventTypeImage = EVENT_TYPES.find(
    (eventType) => eventType.value === event.type
  );

  const joinEventFunction = (e, type) => joinEvent(e, type);
  const leaveEventFunction = (e) => leaveEvent(e);

  const isAlreadyAdminForEvent = () => {
    const userId = USER_CONNECTED.id;
    const attendees = event.links.attendees;
    return (
      attendees &&
      Object.keys(attendees)
        .map((key) => {
          const attendee = attendees[key];
          return key === userId && attendee.isAdmin === true;
        })
        .includes(true)
    );
  };

  console.log(event);

  const changeRole = event.isFollowed && (
    <button
      disabled={mode == "getOnly" ? true : false}
      className={`${'hidden sm:flex '}bg-lime-600 p-2 rounded-lg items-center focus:outline-none`}
    >
      <i
        style={{ fontSize: 13 }}
        className={`fa ${
          isAlreadyAdminForEvent() ? "fa-user-secret" : "fa-user-circle"
        } text-sm text-gray-50 dark:text-gray-50`}
      ></i>
      <p style={{ fontSize: 11 }} className="ml-2 text-gray-50  dark:text-gray-50">
        {isAlreadyAdminForEvent() ? "Déjà  admin" : "Déjà  contributeur-rice"}
      </p>
    </button>
  );

  const isFollowedOrNot = event.isFollowed ? (
    <Button
      disabled={mode == "getOnly" ? true : false}
      fullWidth={true}
      className="bg-gray-900"
      variant="gradient"
      onClick={(e) => {
        e.stopPropagation();
        leaveEventFunction(event._id.$id);
      }}
    >
      Retirer ma participation
    </Button>
  ) : (
    <Button
      disabled={mode == "getOnly" ? true : false}
      fullWidth={true}
      className="bg-gray-900"
      variant="gradient"
      onClick={(e) => {
        e.stopPropagation();
        joinEventFunction(event._id.$id, "connect");
      }}
    >
      Participer
    </Button>
  );

  const organizer = event.organizer && (
    <>
      <div className="flex flex-wrap md:flex-wrap-reverse space-x-1">
        {Object.keys(event.organizer).map((key, index) => (
          <ChipWithAvatar
            name={event.organizer[key].name}
            image={BASE_URL + event.organizer[key].profilThumbImageUrl}
            defaultImage={defaultImage}
            backgroundColor={"dark:bg-gray-600"}
          />
        ))}
      </div>
    </>
  );

  const eventDateReccurencyOrNot = event.openingHours ? (
    <>
      <CardTile
        title={"Événement récurrent"}
        leading={
          <MiniCardIcon icon={"fa fa-calendar"} backgroundColor={"bg-gray-300 dark:bg-gray-700 "}/>
        }
        trailing={
          <h5 className="text-center mt-1"> Chaque </h5>
        }
      >
        <div className="w-auto h-auto items-center justify-center">
          <div className="grid grid-cols-4 gap-2">
            {event.openingHours.map(
              (day, index) =>
                day &&
                day.dayOfWeek &&
                day.hours && (
                  <div
                    key={index}
                    className="flex flex-col w-auto h-auto bg-gray-300 dark:bg-gray-700 rounded-lg p-1 items-center"
                  >
                    <p className="text-gray-700 dark:text-gray-200 text-xs font-semibold">
                      {convertToFrenchDay1(day.dayOfWeek)}
                    </p>
                    <div className="text-center">
                      {day.hours.map((hour, hourIndex) => (
                        <div key={hourIndex} className="mb-1">
                          <span className="text-xs dark:text-gray-300">
                            {hour.opens}
                          </span>
                          <span className="mx-1 text-xs dark:text-gray-300">-</span>
                          <span className="text-xs dark:text-gray-300">
                            {hour.closes}
                          </span>
                        </div>
                      ))}
                    </div>
                  </div>
                )
            )}
          </div>
        </div>
      </CardTile>
    </>
  ) : (
    <>
      <div className="flex justify-between items-center">
        <DateRenderer date={startDate} isAllDate={true} />
        <p className="text-gray-800 dark:text-white"> Au </p>
        <DateRenderer date={endDate} isAllDate={true} />
      </div>
    </>
  );

  const onInvited = () => {
    const invited = [];
    if (event.links && event.links.attendees) {
      for (const key in event.links.attendees) {
        const attendee = event.links.attendees[key];

        if (
          attendee &&
          attendee.hasOwnProperty("isInviting") &&
          attendee.isInviting === true
        ) {
          invited.push(attendee);
        }
      }
    }

    return (
      <>
        <div className="absolute right-4 mt-2 inline-flex items-center cursor-pointer justify-center w-8 h-8 text-gray-300 transition-colors duration-150 bg-gray-50 dark:bg-gray-600 rounded-full focus:shadow-outlin">
          <i
            data-tooltip-id="invite-tooltip"
            data-tooltip-content="invité-e(s)"
            className="fa-solid fa-envelope text-gray-500 dark:text-gray-300 relative"
          >
            {invited.length !== 0 ? (
              <span
                style={{ fontSize: 10 }}
                className="bg-red-500 text-gray-200 rounded-full w-4 h-4 flex items-center justify-center absolute -top-3 text-sm -right-2"
              >
                {invited.length}
              </span>
            ) : null}
          </i>
          <Tooltip
            className="dark:bg-gray-700 -mt-4 cursor-pointer"
            id="invite-tooltip"
          />
        </div>
      </>
    );
  };

  const onGoing = () => {
    const participated = [];

    if (event.links && event.links.attendees) {
      for (const key in event.links.attendees) {
        const attendee = event.links.attendees[key];
        if (
          attendee &&
          attendee.hasOwnProperty("isInviting") &&
          attendee.isInviting === true
        ) {
          return;
        } else {
          participated.push(attendee);
        }
      }
    }

    return (
      <>
        <div className="mt-1">
          <p className="text:gray-800 dark:text-gray-50 mb-2">
            {participated.length} se rendre à cet événement
          </p>
          <div className="border-b border-gray-600"></div>
          <div className="flex space-x-2 cursor-pointer mt-1">
            <CircleAvatarStack participated={participated} />
          </div>
        </div>
      </>
    );
  };

  return (
    <>
      <div className="w-full h-full">
        <CoverAndProfilImage
          coverImage={BASE_URL + event.profilMediumImageUrl}
          profilImage={eventTypeImage && eventTypeImage.image}
          defaultImage={defaultImage}
          altCover={event.name}
        />

        {onInvited()}

        <div className="mt-14 flex-col space-y-4">
          <div className="flex-col mt-5">
            <h1 className="dark:text-gray-50 font-semibold text-3xl">
              {event.name}
            </h1>

            {event.address && (
              <>
                <div className="flex-wrap items-center space-x-2">
                  <div className="flex space-x-1">
                    <div className="dark:bg-gray-600 w-8 h-8 rounded-lg p-1 flex items-center justify-center">
                      <i className="fa-solid text-xs fa-location-dot text-gray-50"></i>
                    </div>

                    <p style={{ fontSize: 12 }} className="text-2xl text-white">
                      {event.address.streetAddress === ""
                        ? event.address.postalCode +
                          " " +
                          event.address.addressLocality
                        : event.address.streetAddress}
                    </p>
                  </div>
                </div>
              </>
            )}
          </div>

          {organizer}

          {eventDateReccurencyOrNot}
        </div>

        <div className="mt-10">
          <p className="text-gray-900 dark:text-gray-50 mb-2">À propos de l'événement</p>
          <div className="border-b border-gray-600"></div>
          <div className="my-4 pb-5 text-justify text-gray-700 dark:text-gray-300">
            <span>
              {" "}
              {event.shortDescription ? (
                <span> {event.shortDescription}</span>
              ) : (
                <span>
                  {" "}
                  L'organisateur·ice de l'événement n'a pas ajouté de
                  description.{" "}
                </span>
              )}{" "}
            </span>
            <p
              style={{ fontSize: 12 }}
              className="flex text-sm text-lime-600 overflow-clip"
            >
              {event.tags &&
                event.tags.map((tag, index) => (
                  <>
                    {" "}
                    <span className="p-1 rounded-xl" key={index}>
                      #{tag}
                    </span>{" "}
                  </>
                ))}
            </p>
          </div>
        </div>

        <CardTile 
          title={"Bienvenue ! Pour participer à l'événement"}
          leading={
            <MiniCardIcon icon={"fa fa-link"} backgroundColor={"bg-gray-300 dark:bg-gray-700"}/>
         }
        >
          <div className="flex justify-between">
              <div className="flex items-center gap-4">
                <Avatar
                  src={BASE_URL + USER_CONNECTED.profilThumbImageUrl}
                  alt="avatar"
                />
                <div>
                  <Typography className="text-gray-700 dark:text-gray-50" variant="h6">
                    {" "}
                    {USER_CONNECTED.name}{" "}
                  </Typography>
                  <Typography
                    variant="small"
                    className="font-normal text-gray-600 dark:text-gray-300"
                  >
                    {USER_CONNECTED.email}
                  </Typography>
                </div>
              </div>
              {changeRole}
            </div>
            <div className="p-2">{isFollowedOrNot}</div>
        </CardTile>
        <br />
        {onGoing()}
        <br />
      </div>
    </>
  );
}

EventDetails.propTypes = {
  event: PropTypes.object.isRequired,
  mode: PropTypes.string,
  joinEvent: PropTypes.func,
  leaveEvent: PropTypes.func,
}

export default EventDetails;
