
import { useContext, useMemo, useState } from "preact/hooks";
import StateContext from "../../../core/context/state-context";
import CustomDrawer from "../../../core/presentation/component/CustomDrawer";
import EventDetails from "./EventDetails";
import {
  getEvents,
  joinEvent,
  leaveEvent,
} from "../../infrastructure/constant/api-endpoints";
import { Switch, Typography } from "@material-tailwind/react";
import CalendarComponent from "./CalendarComponent";

const MyCalendar = () => {
  const { events } = useContext(StateContext);
  const [selectedEvent, setSelectedEvent] = useState({});
  const [isOpenDrawer, openAndCloseDrawer] = useState(false);
  const [isRecurency, setIsRecurency] = useState(false);
  const currentDate = new Date().toISOString().split("T")[0];

  const openEventDetailsFunction = () => openAndCloseDrawer(!isOpenDrawer);

  const handleSwitchValue = (e) => setIsRecurency(e.target.checked);

  const fetchData = async () => {
    try {
      const eventsList = await getEvents(currentDate);
      events.signal.value = eventsList.data;
    } catch (error) {
      console.error("Error fetching events: ", error);
    }
  };

  const joinEventFunction = async (e, type) => {
    await joinEvent(e, type).then(async (res) => {
      if (res.data.result) {
        await fetchData();
        var event = events.signal.value.filter(
          (ev) => ev._id.$id === selectedEvent._id.$id
        )[0];
        setSelectedEvent(event);
      }
    });
  };

  const leaveEventFunction = async (e) => {
    await leaveEvent(e).then(async (res) => {
      if (res.data.result) {
        await fetchData();
        var event = events.signal.value.filter(
          (ev) => ev._id.$id === selectedEvent._id.$id
        )[0];
        setSelectedEvent(event);
      }
    });
  };

  const eventsWithoutRecurrence = useMemo(() => {
    return events.signal.value
      .filter((e) => !e.recurrency)
      .map((event) => ({
        title: event.name,
        start: event.startDateTime,
        end: event.endDateTime,
        eventOne: event,
      }));
  }, [events]);

  const generateEventsByDay = (eventReccurency, numYears = 2) => {
    const events = [];
  
    const currentDate = new Date();
  
    const eventsRecurency = useMemo(() => {
      return eventReccurency.filter((event) => event.recurrency);
    }, [eventReccurency]);
  
    for (let year = 0; year < numYears; year++) {
      for (let month = 0; month < 12; month++) {
        const daysInMonth = new Date(
          currentDate.getFullYear() + year,
          month + 1,
          0
        ).getDate();
  
        for (let day = 1; day <= daysInMonth; day++) {
          eventsRecurency.forEach((event) => {
            event.openingHours.forEach((openingHour) => {
              const dayOfWeek = getDayOfWeekAbbreviation(
                new Date(currentDate.getFullYear() + year, month, day).getDay()
              );
  
              if (dayOfWeek === openingHour.dayOfWeek) {
                openingHour.hours.forEach((hour) => {
                  const opensTime = new Date(
                    currentDate.getFullYear() + year,
                    month,
                    day,
                    getHour(hour.opens),
                    getMinute(hour.opens),
                    0
                  );
                  const closesTime = new Date(
                    currentDate.getFullYear() + year,
                    month,
                    day,
                    getHour(hour.closes),
                    getMinute(hour.closes),
                    0
                  );
  
                  events.push({
                    title: event.name,
                    start: opensTime.toISOString(),
                    end: closesTime.toISOString(),
                    eventOne: event,
                  });
                });
              }
            });
          });
        }
      }
    }
  
    return events;
  };

  const getDayOfWeekAbbreviation = (dayIndex) => {
    const daysAbbreviation = ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"];
    return daysAbbreviation[dayIndex];
  };
  
  const getHour = (time) => {
    return parseInt(time.split(":")[0], 10);
  };
  
  const getMinute = (time) => {
    return parseInt(time.split(":")[1], 10);
  };

  const CustomEvent = ({ event }) => {
    return (
      <div
        onClick={(e) => {
          e.stopPropagation();
          setSelectedEvent(event.extendedProps.eventOne);
          openEventDetailsFunction();
        }}
        className="ml-3 bg-lime-600 cursor-pointer"
      >
        <strong className="ml-2">{event.title}</strong>
      </div>
    );
  };

  const CustomEventReccurence = ({ event }) => {
    const truncateStyle = {
      whiteSpace: "nowrap",
      overflow: "hidden",
      textOverflow: "ellipsis",
    };

    return (
      <div
        onClick={(e) => {
          e.stopPropagation();
          setSelectedEvent(event.extendedProps.eventOne);
          openEventDetailsFunction();
        }}
        className="flex flex-row space-x-1 items-center cursor-pointer"
        style={truncateStyle}
      >
        <i
          className="fa-solid fa-circle text-lime-600"
          style={{ fontSize: 8 }}
        ></i>
        <strong className=""> {event.title} </strong>
      </div>
    );
  };

  const allEventsReccurence = generateEventsByDay(events.signal.value);

  return (
    <div className="mt-16 md:mt-0">
      <div className="fixed right-4 top-5">
        <Switch
          checked={isRecurency}
          onChange={handleSwitchValue}
          ripple={false}
          className="md:h-full md:w-full sm:h-8 sm:w-8 dark:checked:bg-lime-600 checked:bg-lime-600 bg-gray-300 dark:bg-gray-700"
          containerProps={{
            className: "md:w-11 md:h-6 sm:w-6 sm:h-3",
          }}
          circleProps={{
            className: "before:hidden left-0.5 border-none",
          }}
          label={
            <Typography
              className={`${"md:font-medium text-gray-700 dark:text-gray-300 md:text-xl text-sm"}`}
            >
              Événement récurrent
            </Typography>
          }
        />
      </div>

        <CalendarComponent
          events={ isRecurency ? allEventsReccurence : eventsWithoutRecurrence}
          eventContent={ isRecurency ? CustomEventReccurence : CustomEvent}
        />
     

      <CustomDrawer
        isOpenDrawer={isOpenDrawer}
        openAndCloseDrawer={openEventDetailsFunction}
        openPageTitle={"Ouvrir la page de l'événement"}
        placement={"right"}
      >
        <EventDetails
          event={selectedEvent}
          joinEvent={joinEventFunction}
          leaveEvent={leaveEventFunction}
        />
      </CustomDrawer>
    </div>
  );
};

export default MyCalendar;
