import { useState, useCallback, useEffect } from 'react';

const useLocalStorage = (key, initialValue) => {
  const parseJSON = (value) => {
    try {
      return value === 'undefined' ? undefined : JSON.parse(value ?? '');
    } catch {
      return undefined;
    }
  };
  const readValue = useCallback(() => {
    if (typeof window === 'undefined') {
      return initialValue;
    }
    try {
      const item = window.localStorage.getItem(key);
      return item ? parseJSON(item) : initialValue;
    } catch (error) {
      return initialValue;
    }
  }, [initialValue, key]);

  const [storedValue, setStoredValue] = useState(() => readValue());
  const setValue = useCallback((value) => {
    try {
      const newValue = value instanceof Function ? value(storedValue) : value;

      window.localStorage.setItem(key, JSON.stringify(newValue));

      setStoredValue(newValue);

      window.dispatchEvent(new Event('local-storage'));
    } catch (error) {
      console.log(error);
    }
  }, [key, storedValue]);

  useEffect(() => {
    setStoredValue(readValue());
  }, [readValue]);

  const handleStorageChange = useCallback((event) => {
    if (event?.key && event.key !== key) {
      return;
    }
    setStoredValue(readValue());
  }, [key, readValue]);

  useEffect(() => {
    window.addEventListener('storage', handleStorageChange);
    window.addEventListener('local-storage', handleStorageChange);

    return () => {
      window.removeEventListener('storage', handleStorageChange);
      window.removeEventListener('local-storage', handleStorageChange);
    };
  }, [handleStorageChange]);

  return [storedValue, setValue];
}

export default useLocalStorage;